""" 
周线策略

周线主要是看趋势。
趋势向上，才考虑买入。

趋势指标：
MACD：这是一个中长期指标。

MACD指标属于大势趋势类指标，它由长期
* 均线DEA
* 短期均线DIF
* 红色能量柱（多头）
* 绿色能量柱（空头）
* 0轴（多空分界线）
五部分组成。


1 当DIF、DEA两数值位于0轴上方时，说明大势处于多头市场，投资者应当以持股为主要策略。
若DIF由下向上与DEA产生交叉，并不代表是一种买入信号，而此时的大盘走势，已是一个短期高点，应当采用高抛低吸的策略。
一般情况下，在交叉信号产生后的第二天或第三天，会有一个回调低点，此刻可以再行买入，达到摊低成本的目的。
若DIF由上向下交叉DEA时，说明该波段上升行情已经结束，通常行情会在交叉信号产生后，有波像样的反弹，已确认短期顶部的形成，此时投资者可以借机清仓出局。
在之后的调整中，利用随机指标KDJ，强弱指标RSI再伺机介入，摊低成本。若DIF第二次由下向上与DEA交叉，预示着将产生一波力度较大的上升行情，
在交叉信号产生后，投资者应当一路持股，直到DIF再次由上向下交叉DEA时，再将所有的股票清仓，就可以扛着钱袋回家休息了。
由于股市行情的变化多端，MACD指标常会与K线走势图呈背离的走势，通常称为熊背离。既K线走势图创出的第二个或第三个高点，MACD指标并不配合出现相应的高点，却出现相反的走势，顶点在逐步降低。
此种现象应引起投资者的警觉，因为它预示着今后将有大跌行情产生，所以投资者宜采用清仓离场的策略，使自己的股票避免被套，资金避免受到损失。

2 当DIF与DEA两指标位于0轴的下方时，说明的大势属于空头市场，投资者应当以持币为主要策略。若DIF由上向下交叉DEA时，会产生一个调整低点。一般情况下，在此之后由一波反弹行情产生，这是投资者一次很好的*仓机会。在中国股市中，还没有建立作空机制，因此股市一旦进入空头市场，投资者最好的策略就是离场观望。投资者可以在股票贬值的同时，使手中的资金得到增值。若DIF由下向上交叉DEA时，会产生的一个高点，投资者应当果断*仓。这种信号的产生，一般以反弹的性质居多。在空头市场中，每次反弹都应当视为出货的最佳良机。尤其需要引起注意的是，若DIF第二次由上向下交叉DEA时，预示着今后会有一波较大的下跌行情产生。投资者应当在交叉信号产生后，坚决清仓出局。通常产生的这段下跌，属于波浪理论中的C浪下跌，是最具杀伤力的一波下跌。只有躲过C浪下跌，才可以说真正在股市中赚到了钱。在空头市场经过C浪下跌以后，偶尔也会发生MACD指标与K线走势图产生背离的现象，通常称为牛背离。既K线走势图出现第二或第三个低点，MACD指标并没有相应的低点产生，却出现一底高过一底的相反走势，这种现象的产生，预示着行情在今后会发生反转走势，投资者应当积极介入，因为市场根本没有风险。

3 当MACD指标作为单独系统使用时，短线可参考DIF走势研判。若DIF由上向下跌穿O轴时可看作大势可能步入空头市场，预示着大势将走弱，应当引起投资者的警觉。在空头市场中，投资者承受的风险高于收益。若MACD由上向下跌空O轴时，确认大势进入空头市场。投资者应采用离场观望的策略，以回避市场风险，使牛市中赚到的利润得到保障。若DIF由下向上穿越O轴时，可看作大势可能步入多头市场。预示着大势将走强，操作上应部分资金参与。若MACD由下向上穿越O轴时确认大势进入多头市场。投资者可以大胆持股，积极介入。在多头市场中，获得的收益高于承担的风险。

4在MACD指标中，红色能量柱和绿色能量柱，分别代表了多头和空头能量的强弱盛衰。它们对市场的反应，要比短期均线DIF在时间上提前。在MACD指标中，能量释入的过程，是一个循序渐*的过程，通常是呈逐渐放大的。在东方哲学中讲求，”阳盛则衰、阴盛则强”。在使用能量柱时，利用红色能量柱结合K线走势图就得出，当K线走势图*乎90度的上升，加之红色能量柱的快速放大，预示着大势的顶部已*。尤其是相邻的两段红色能量柱产生连片时，所爆发的行情将更加迅猛。反之，在空头市场中，这种现象也成立。在熟悉了这种操作手法后，对投资者逃顶和抄底将大有益处。

5在使用MACD指标过程中，有两点需要注意：
第一，MACD指标对于研判短期顶部和底部，并不一定可信，只有结合中期乖离率和静态中的ADR指标，才可以判定。
第二，利用周线中的MACD指标分析比日线的MACD指标效果好。

总之，在使用MACD指标时必须判定市场的属性。即市场是多头市场，还是空头市场。根据不同的市场属性，采取不同的操作策略，以回避风险，保障利润的目的。具体操作中，MACD的黄金交叉一般是重要的买入时机。
首先，就其要点分析，当DIF和MACD两线在0轴之下且较远时由下行转为走*，且快线DIF上穿慢线MACD形成的金叉是较佳的短线买入时机，但必须注意DIF和MACD距离0轴远*的判断主要根据历史记录作为参考。而发生在0轴之上的金叉则不能离0轴太远，否则其可靠性将大大降低。比较倾向于在红海洋既红柱连成一片区，在0轴上方DIF正向交叉MACD形成金叉，其中线可靠性较好。同时这也符合强势市场机会多，弱势市场难赚钱的股市道理。

买卖原则为：
1.DIF、DEA均为正，DIFF向上突破DEA，买入信号参考。
2.DIF、DEA均为负，DIFF向下跌破DEA，卖出信号参考。
3.DEA线与K线发生背离，行情可能出现反转信号。
4.MACD的值从正数变成负数，或者从负数变成正数并不是交易信号，因为它们落后于市场。


使用MACD指标所应当遵循的基本原则：
⒈当DIF和DEA处于0轴以上时，属于多头市场。
⒉当DIF和DEA处于0轴以下时，属于空头市场。
⒊柱状线收缩和放大。
⒋形态和背离情况。
5 牛市中指标将失真。

用DIF的曲线形状进行分析，主要是利用指标相背离的原则。
如果DIF的走向与股价走向相背离，则是采取具体行动的时间。但是，根据以上原则来指导实际操作，准确性并不能令人满意。
经过实践、摸索和总结，综合运用5日、10日均价线，5日、10日均量线和MACD，其准确性大为提高。

8.当 MACD 与K 线图的走势出现背离时，应该视为股价即将反转的信号，必须注意盘中走势。


! 缺点
1. 由于MACD是一项中、长线指标，买进点、卖出点和最低价、最高价之间的价差较大。
当行情忽上忽下幅度太小或盘整时，按照信号进场后随即又要出场，买卖之间可能没有利润，也许还要赔点价差或手续费。
* 周K线MACD指标对中长线转折的判断的准确性较高，可以作为中长线投资者的首选参考指标。

2. MACD的移动相当缓和。
一两天内涨跌幅度特别大时，MACD来不及反应。
比较行情的移动有一定的时间差，所以一旦行情迅速大幅涨跌，MACD不会立即产生信号，此时，MACD无法发生作用。



当MACD数值超过5且指标出现红柱后的第二个星期开盘时买入，在超过数值-5且指标变成绿柱后的第二个星期开盘时卖出。
! 不可能超过-5。
MACD下跌。

1. 大趋势向上
2. 买的便宜
3. 优质资产
4. 小市值

! 低位金叉
一、“MACD低位两次金叉”
首先是“MACD低位两次金叉”出暴利机会。MACD指标的要素主要有红色柱、绿色柱、DIF指标、DEA指标。其中，当DIF、DEA指标处于O轴以下的时候，如果短期内（8或13个交易日内）连续发生两次金叉，则发生第二次金叉的时候，可能发生暴涨。

使用“MACD低位二次金叉”寻找短线暴涨股，需注意下列事项：
（一）MACD低位一次金叉的，未必不能出暴涨股，但“MACD低位二次金叉”出暴涨股的概率和把握更高一些。
（二）“MACD低位二次金叉”出暴涨股的概率之所以更高一些，是因为经过“第一次金叉”之后，空头虽然再度小幅进攻、造成又一次死叉，但是，空头的进攻在多方的“二次金叉”面前，遭遇溃败。从而造成多头力量的喷发。
（三）“MACD低位二次金叉”，如果结合K线形态上的攻击形态研判，则可信度将提高，操盘手盘中将更容易下决心介入。形成了“两阳吃一阴”，当天并且温和放量，综合研判的可信度明显增加。也即：“MACD低位二次金叉”和K线形态、量价关系可以综合起来考虑，以增加确信度。


! 高位金叉
"""


from pyfund import constants, core
from pyfund.strategy import IStrategy, Operate
from rich import print

class Strategy(IStrategy):
    # ! 策略自己控制仓位

    keys = [
        'operate',
        'irr',
        constants.I_DATE,
        constants.I_OPEN,
        constants.I_CLOSE,
        constants.I_CR_CLOSE,
        # constants.I_VOLUME,
        # constants.I_CR_VOLUME,
        constants.I_CR_MACD,
        constants.I_MACD,
        constants.I_MACDX,
        constants.I_DEA,
        constants.I_DIF,
        # constants.I_KDJD,
        # constants.I_CR_KDJD,
        # constants.I_KDJJ,
        constants.I_MA5,
        constants.I_CR_MA5,
        constants.I_MA10,
        constants.I_CR_MA10,
        constants.I_MA20,
        constants.I_CR_MA20,
        constants.I_PE,
        constants.I_PB,
        constants.I_ROE,
        constants.I_BOLL
    ]

    counter = 0
    low_macd = []

    def sell_all(self, data: core.Data):
        if self.num == 0:
            return Operate.NONE
    
        macd = data.get(constants.I_MACD)
        cr_macd = data.get(constants.I_CR_MACD)
        macdx = data.get(constants.I_MACDX)
        date =  data.get(constants.I_DATE)
        dea =  data.get(constants.I_DEA)
        openx = data.get(constants.I_OPEN)

        if macdx == -1 and dea > 0: # 低位金叉
            # print(date, '高位死叉，清仓')
            return Operate.SELL_ALL

        close = data.get(constants.I_CLOSE)
        if self.max_boll and self.num > 0:
            max_close = max(self.closes)
            # 跌破最大收益20%
            if close <= (max_close - self.fall_from_max*(max_close - self.cost)):
                # print(data.get(constants.I_DATE), 'max清仓')
                return Operate.SELL_ALL
        
        # 亏4%卖么？
        # 买入后，亏损4%，直接清仓
        # ! 设定一个条件单，卖出条件单就好
        if (openx - self.cost)/self.cost <= -0.04:
            # print(data.get(constants.I_DATE), '0.04清仓')
            return Operate.SELL_ALL
        

    def sell(self, data: core.Data):
        return Operate.NONE
    # 买入多少份，由策略制定者决定
    # 标记is_full 为满仓标记
    # 是否满仓，交给人类自己判断
    def buy_all(self, data):
        # if self.is_full: # 满仓
        #     return Operate.NONE
        

        if data.get(constants.I_CR_MA5) <= 0 or data.get(constants.I_CR_MA10) <= 0:
            return Operate.NONE

        macd = data.get(constants.I_MACD)
        cr_macd = data.get(constants.I_CR_MACD)
        macdx = data.get(constants.I_MACDX)
        date =  data.get(constants.I_DATE)
        dea =  data.get(constants.I_DEA)
    
        if macdx == 1 and dea > 0: # 低位金叉
            # print(date, '[green]高位金叉[/green]，满仓')
            return Operate.BUY_ALL
    
        if macdx == 1 and dea < 0: # 低位金叉
            # print(date, '[red]低位金叉[/red]')
            if self.counter == 0:
                self.low_macd.append(dea)
                self.counter += 1
            elif self.counter == 2:
                if len(self.low_macd) == 1:
                    if self.low_macd[0] <dea:
                        # print(date, '[red]二次低位金叉[/red]，满仓')
                        self.counter = 0
                        self.low_macd.clear()
                        return Operate.BUY_ALL
                    else:
                        self.counter = 1
                        self.low_macd.clear()
                        self.low_macd.append(dea)
                        return Operate.NONE

        if macdx == -1 and dea < 0: # 低位死叉
            # print(date, '[red]低位死叉[/red]')
            self.counter += 1
            return Operate.NONE
        if macdx == -1 and dea > 0: # 低位死叉
            # print(date, '[red]高位死叉[/red]')
            self.counter += 1
            return Operate.NONE

    def buy(self, data):
        macdx = data.get(constants.I_MACDX)
        dea =  data.get(constants.I_DEA)
        date =  data.get(constants.I_DATE)

        return Operate.NONE
